/*----------------------------------------------------------------------------
 *      main Template for CMSIS RTE C/C++ Project
 *----------------------------------------------------------------------------
 *      Name:    main.c
 *      Purpose: Generic main program body including main() function
 *      Rev.:    1.0.0
 *----------------------------------------------------------------------------*/
/*******************************************************************************
* Copyright (c) 2015 ARM Ltd. and others
* All rights reserved. This program and the accompanying materials
* are made available under the terms of the Eclipse Public License v1.0
* which accompanies this distribution, and is available at
* http://www.eclipse.org/legal/epl-v10.html
*
* Contributors:
* ARM Ltd and ARM Germany GmbH - file template
*******************************************************************************/

#include "RTE_Components.h"             // Component selection
#include "cmsis_os2.h"                   // CMSIS RTOS header file
#include "system_iMX7D_M4.h"
#include "board.h"
#include "device_imx.h"
#include "gpio_imx.h"
#include <stdio.h>


/*----------------------------------------------------------------------------
 *      LEDs
 *---------------------------------------------------------------------------*/

typedef struct _GPIO_PIN {                  /* GPIO Pin identifier */
  GPIO_Type   *port;
  uint32_t      pin;
} GPIO_PIN;

static const GPIO_PIN LED_PIN[] = {         /* LED GPIO Pins */
  { GPIO1, 7  },
  { GPIO1, 6  },
  { GPIO1, 5  },
  { GPIO1, 4  }
};

#define LED_COUNT (sizeof(LED_PIN)/sizeof(GPIO_PIN))

int32_t LED_Initialize (void) {

  /* GPIO: Set Mux to IO Mode */
  IOMUXC_LPSR_SW_MUX_CTL_PAD_GPIO1_IO04 = IOMUXC_LPSR_SW_MUX_CTL_PAD_GPIO1_IO04_MUX_MODE(0);
  IOMUXC_LPSR_SW_MUX_CTL_PAD_GPIO1_IO05 = IOMUXC_LPSR_SW_MUX_CTL_PAD_GPIO1_IO05_MUX_MODE(0);
  IOMUXC_LPSR_SW_MUX_CTL_PAD_GPIO1_IO06 = IOMUXC_LPSR_SW_MUX_CTL_PAD_GPIO1_IO06_MUX_MODE(0);
  IOMUXC_LPSR_SW_MUX_CTL_PAD_GPIO1_IO07 = IOMUXC_LPSR_SW_MUX_CTL_PAD_GPIO1_IO07_MUX_MODE(0);

  /* Configure GPIO pins */
  gpio_init_config_t initConfig;
  initConfig.direction = gpioDigitalOutput;
  initConfig.interruptMode = gpioNoIntmode;

  initConfig.pin = 4;
  GPIO_Init(GPIO1, &initConfig);
  initConfig.pin = 5;
  GPIO_Init(GPIO1, &initConfig);
  initConfig.pin = 6;
  GPIO_Init(GPIO1, &initConfig);
  initConfig.pin = 7;
  GPIO_Init(GPIO1, &initConfig);

  return 0;
}

int32_t LED_On (uint32_t num) {
	GPIO_WritePinOutput(LED_PIN[num].port, LED_PIN[num].pin, gpioPinClear);
  return 0;
}

int32_t LED_Off (uint32_t num) {
  GPIO_WritePinOutput(LED_PIN[num].port, LED_PIN[num].pin, gpioPinSet);
  return 0;
}


/*----------------------------------------------------------------------------
 *      Thread A
 *---------------------------------------------------------------------------*/
void threadA (void *argument) {
  int32_t led_num = 0;
  uint32_t led_dir = 0;
  for (;;) {
    osDelay(50);
    LED_On(led_num);                     // Turn specified LED on
    osDelay(50);
    LED_Off(led_num);                    // Turn specified LED off
    if (led_dir == 0) {
      led_num++;                       // Change LED number
      if (led_num >= LED_COUNT - 1) {
        led_dir = 1;                 // Restart with first LED
      }
    } else {
      led_num--;                       // Change LED number
      if (led_num <= 0) {
        led_dir = 0;                 // Restart with first LED
      }
    }
  }
}


int main(void)
{
  /* Board specific RDC settings */
  BOARD_RdcInit();

  /* Board specific clock settings */
  BOARD_ClockInit();

  LED_Initialize();

  // System Initialization
  SystemCoreClockUpdate();
  // ...
  osKernelInitialize();                 // Initialize CMSIS-RTOS

  osThreadNew(threadA, NULL, NULL);

  if (osKernelGetState() == osKernelReady) {
    osKernelStart();                    // Start thread execution
  }

  while (1);
}

